package cz.cuni.jagrlib;

import cz.cuni.jagrlib.iface.*;

/**
 * Material object for Phong shading model (with constant or Schlick K() term).
 *
 * @version 0.26 $Rev: 476 $ $Date: 2007-06-03 03:11:16 +0200 (Sun, 03 Jun 2007) $ $Author: pepca $
 * @since   0.12
 * @see     <a href="../../../../cz/cuni/jagrlib/PhongMaterial.java">PhongMaterial.java</a>
 */
public class PhongMaterial extends DefaultProperty
{
  //==========================================================================
  //  Data:

  /** Original color of the solid surface. */
  public double[] origColor;

  /** Actual color (set by {@link LightModel#setColor}). */
  public double[] color;

  /** Ambient-light coefficient. */
  public double ka;

  /** Diffuse reflection coefficient. */
  public double kd;

  /** Specular reflection coefficient. */
  public double ks;

  /** Specular reflection exponent. */
  public int h;

  /** Transmission (refraction) coefficient. */
  public double kt;

  /** Relative index of refraction. */
  public double n;

  /** Limit cos-value for total reflection. */
  public double cosTotal;

  /** Schlick blending coefficient ({@code 0.0} .. non-Schlick, {@code 1.0} .. Schlick). */
  public double sch;

  /** Default constructor. */
  public PhongMaterial ()
  {
    origColor = new double[] { 0.4, 0.25, 0.1 };
    ka        = 0.2;
    kd        = 0.6;
    ks        = 0.3;
    h         = 16;
    kt        = 0.0;
    n         = 1.0;
    cosTotal  = 0.0;
    sch       = 0.0;
  }

  /**
   * Sets the given property.
   *
   * @param  key Key string.
   * @param  value The new value.
   */
  @Override
  public void set ( String key, Object value )
  {
    if ( key == null || value == null ) return;

    if ( key.compareTo( LightModel.MATERIAL_COLOR ) == 0 &&
         value instanceof double[] )
    {
      origColor = (double[])value;
      return;
    }

    if ( key.compareTo( LightModel.MATERIAL_KA ) == 0 )
    {
      ka = doubleProperty( value, ka );
      return;
    }

    if ( key.compareTo( LightModel.MATERIAL_KD ) == 0 )
    {
      kd = doubleProperty( value, kd );
      return;
    }

    if ( key.compareTo( LightModel.MATERIAL_KS ) == 0 )
    {
      ks = doubleProperty( value, ks );
      return;
    }

    if ( key.compareTo( LightModel.MATERIAL_H ) == 0 )
    {
      h = intProperty( value, h );
      return;
    }

    if ( key.compareTo( LightModel.MATERIAL_KT ) == 0 )
    {
      kt = doubleProperty( value, kt );
      return;
    }

    if ( key.compareTo( LightModel.MATERIAL_N ) == 0 )
    {
      n = doubleProperty( value, n );
      cosTotal = n > 1.0 ? (Math.sqrt( n * n - 1.0 ) / n) : 0.0;
      return;
    }

    if ( key.compareTo( "Sch" ) == 0 )
    {
      sch = doubleProperty( value, sch, 0.0, 1.0 );
      return;
    }

  }

  /**
   * Gets the given property.
   *
   * @param  key Key string.
   * @return The actual value or {@code null}.
   */
  @Override
  public Object get ( String key )
  {
    if ( key == null ) return null;

    if ( key.compareTo( LightModel.MATERIAL_COLOR ) == 0 ) return origColor;
    if ( key.compareTo( LightModel.MATERIAL_KA )    == 0 ) return ka;
    if ( key.compareTo( LightModel.MATERIAL_KD )    == 0 ) return kd;
    if ( key.compareTo( LightModel.MATERIAL_KS )    == 0 ) return ks;
    if ( key.compareTo( LightModel.MATERIAL_H )     == 0 ) return h;
    if ( key.compareTo( LightModel.MATERIAL_KT )    == 0 ) return kt;
    if ( key.compareTo( LightModel.MATERIAL_N )     == 0 ) return n;
    if ( key.compareTo( "Sch" )                     == 0 ) return sch;

    return null;
  }
}
