package cz.cuni.jagrlib.iface;

import cz.cuni.jagrlib.MicroFacet;

/**
 * Abstract light source in 3D scene.
 *
 * @version 0.26 $Rev: 434 $ $Date: 2007-05-02 03:57:09 +0200 (Wed, 02 May 2007) $ $Author: pepca $
 * @since   0.11
 * @see     <a href="../../../../../cz/cuni/jagrlib/iface/LightSource.java">LightSource.java</a>
 * @see     LightModel
 * @see     RTScene
 */
public interface LightSource extends Property
{
  /** Property - [primary] surface [diffuse] color. */
  String COLOR = "Color";

  /**
   * Sets intersectable object (for light-source of visibility: {@link #lights(MicroFacet)}, ..).
   */
  Intersectable setIntersectable ( Intersectable inter );

  /**
   * Lighting test from the source to the given {@link MicroFacet}.
   *
   * @param  mf The checked {@link MicroFacet} (point on surface of some solid).
   * @return {@code true} if the light source lights on the given point.
   * @see    #getIntensity
   * @see    #getDirection
   */
  boolean lights ( MicroFacet mf );

  /**
   * Lighting test from the source to the given {@link MicroFacet}.
   * Super-sampling variant.
   *
   * @param  mf The checked {@link MicroFacet} (point on surface of some solid).
   * @param  order Order of this sample inside the actual pixel.
   * @param  total Total number of samples in the actual pixel.
   * @return {@code true} if the light source lights on the given point.
   * @see    #getIntensity
   * @see    #getDirection
   */
  boolean lights ( MicroFacet mf, int order, int total );

  /**
   * Returns light intensity coming from the source to the given {@link MicroFacet}.
   *
   * @param  mf The {@link MicroFacet} (point on surface of some solid).
   * @param  color Pre-allocated {@code double[C]} array to hold result - light intensity
   *               coming from the source to the given point ({@code C} can be either 1
   *               /for monochrome computing/, 3 /for RGB color system/ or greater /for more
   *               complex spectral color sampling/).
   * @return {@code true} if the light source can light on the given point (<b>cannot</b>
   *         replace the {@code lights()} call!).
   * @see    #lights
   * @see    #getDirection
   */
  boolean getIntensity ( MicroFacet mf, double[] color );

  /**
   * Returns light intensity coming from the source to the given {@link MicroFacet}.
   * Super-sampling variant.
   *
   * @param  mf The {@link MicroFacet} (point on surface of some solid).
   * @param  order Order of this sample inside the actual pixel.
   * @param  total Total number of samples in the actual pixel.
   * @param  color Pre-allocated {@code double[C]} array to hold result - light intensity
   *               coming from the source to the given point ({@code C} can be either 1
   *               /for monochrome computing/, 3 /for RGB color system/ or greater /for more
   *               complex spectral color sampling/).
   * @return {@code true} if the light source can light on the given point (<b>cannot</b>
   *         replace the {@code lights()} call!).
   * @see    #lights
   * @see    #getDirection
   */
  boolean getIntensity ( MicroFacet mf, int order, int total, double[] color );

  /**
   * Returns light direction between the source and the given {@link MicroFacet}.
   *
   * @param  mf The {@link MicroFacet} (point on surface of some solid).
   * @param  dir Pre-allocated array to hold the result - <b>normalised</b> direction vector
   *             from the {@link MicroFacet} to the light source.
   * @return {@code false} if the light source is not directional (thus {@code dir}
   *         is irrelevant).
   * @see    #lights
   * @see    #getIntensity
   */
  boolean getDirection ( MicroFacet mf, double[] dir );

  /**
   * Returns light direction between the source and the given {@link MicroFacet}.
   * Super-sampling variant.
   *
   * @param  mf The {@link MicroFacet} (point on surface of some solid).
   * @param  order Order of this sample inside the actual pixel.
   * @param  total Total number of samples in the actual pixel.
   * @param  dir Pre-allocated array to hold the result - <b>normalised</b> direction vector
   *             from the {@link MicroFacet} to the light source.
   * @return {@code false} if the light source is not directional (thus {@code dir}
   *         is irrelevant).
   * @see    #lights
   * @see    #getIntensity
   */
  boolean getDirection ( MicroFacet mf, int order, int total, double[] dir );

}
