package cz.cuni.jagrlib.worker;

import cz.cuni.jagrlib.*;
import cz.cuni.jagrlib.iface.*;
import cz.cuni.jagrlib.reg.RegPiece;

/**
 * Worker for Euler-constructed Brep scene rendered by OpenGL.
 *
 * @author  PE
 * @version 0.26 $Rev: 867 $ $Date: 2009-03-16 09:03:05 +0100 (Mon, 16 Mar 2009) $ $Author: pepca $
 * @see     <a href="../../../../../cz/cuni/jagrlib/worker/EulerWorker.java">EulerWorker.java</a>
 */
public class EulerWorker extends Piece implements Worker
{
  //==========================================================================
  //  Data:

  /** Redraw period in milliseconds (or 0 if no periodic redraw has to be done). */
  protected int period   = 10;

  /** Continue flag for regular redraw loop. */
  protected boolean cont = true;

  /** Accelerator reference to the viewer window. */
  protected GraphicsViewer window;

  /** Implemented construction modes/scenes. */
  protected static enum Constructions
  {
    /** Tetrahedron. */
    TETRAHEDRON,
    /** Cube. */
    CUBE,
    /** Graph of {@code R->R^2} function. */
    FUNCTION_GRAPH,
    /** Euler operators read from XML file. */
    XML_SCRIPT,
  }

  /** Current construction mode/scene. */
  protected Constructions construction = Constructions.TETRAHEDRON;

  //==========================================================================
  //  Support:

  /**
   * Constructs Brep database using Euler operators.
   */
  protected void brepConstruction ()
  {
    final EulerOperators euler = (EulerOperators)getInterface( PL_INPUT,  IFACE + "EulerOperators" );
    final Brep           brep  = (Brep)          getInterface( PL_OUTPUT, IFACE + "Brep" );
    if ( euler == null ||
         brep  == null ) return;

    final int context = Brep.CTX_DEFAULT;

    brep.setEdgeMode( true );   // winged-edge mode set
    int color = brep.createAttribute( Brep.FACE, context, Brep.COLOR, Brep.ATTR_RGB_3F );
    euler.setContext( context );
    int[] c1 = new int[ 3 ];    // temp arrays for Euler constructors
    int[] c2 = new int[ 3 ];
    int[] c3 = new int[ 3 ];

    switch ( construction )
    {
      case CUBE:
        // !!! TODO: finish this construction !!!
        break;

      default:            // tetrahedron
        euler.Mvfs( c1 );
        brep.setVertexCoords( context, c1[0], new double[] { -1.,  -1., -1., 1. } );
        euler.Mve( c2, c1[0], c1[1], c1[1] );
        brep.setVertexCoords( context, c2[0], new double[] {  1.,  -1., -1., 1. } );
        // !!! TODO: finish this construction !!!
    }

    /*
    brep.setAttribute( color, faceN, new float[] { 1.0f, 1.0f, 0.0f } );
    brep.commit();
    */
  }

  //==========================================================================
  //  Executive routine:

  /**
   * Executive routine of the working thread.
   */
  public void run ()
  {
    window = (GraphicsViewer)getInterface( PL_WINDOW, IFACE+"GraphicsViewer" );
    if ( window == null ) return;

    brepConstruction();

    Semaphore sem;
    window.set( GraphicsViewer.WINDOW_TITLE, "GL Graphics" );

      // preview thread:
    GraphicsViewer.PreviewThread runIt = new GraphicsViewer.PreviewThread( window );
    runIt.safeStart();

    if ( period > 0 )
    {
      sem = new Semaphore( 0 );
      window.addSemaphore( sem );

        // timer thread:
      RedrawTimer timer = new RedrawTimer();
      timer.startTimer( window, period );

      sem.semWait( 3600000L );

      timer.stopTimer();
    }
  }

  //==========================================================================
  //  Interface Property:

  public final static String SOLID   = "Solid";
  public final static String PERIOD  = "Redraw period";

  /**
   * Sets the given property.
   *
   * @param  key Key string.
   * @param  value The new value.
   */
  @Override
  public void set ( String key, Object value )
  {
    if ( key == null || value == null ) return;

      // SOLID:
    if ( key.compareTo(SOLID) == 0 )
    {
      construction = enumProperty( value, construction );
      return;
    }
      // PERIOD:
    if ( key.compareTo(PERIOD) == 0 )
    {
      period = intProperty( value, period, 0, 60000 );
      return;
    }
  }

  /**
   * Gets the given property.
   *
   * @param  key Key string.
   * @return The actual value or {@code null}.
   */
  @Override
  public Object get ( String key )
  {
    if ( key == null ) return null;

    if ( key.compareTo(SOLID) == 0 )
      return construction.ordinal();
    if ( key.compareTo(PERIOD) == 0 )
      return period;

    return null;
  }

  //==========================================================================
  //  Module registration:

  /** Object name. */
  private final static String NAME = "EulerWorker";

  /** Object template identifier. */
  protected final static String TEMPLATE_NAME = "WorkerForEulerBrepScene";

  /** Object description. */
  private final static String DESCRIPTION = "Worker for Euler-constructed Brep scene rendered using OpenGL.";

  /** Object category. */
  protected final static String CATEGORY = C_3D + '.' + C_RENDER + '.' + C_WORKER;

  /**
   * General-purpose registration routine.
   * Sets all plugs, strings, etc. to the given Template.
   */
  public static int setTemplate ( Template t, int ord )
  {
    if ( t != null && ord <= 0 )
    {
      t.setRegStrings( NAME, TEMPLATE_NAME, CATEGORY, DESCRIPTION );
        // Plugs:
      t.newOutputPlug(    PL_INPUT,  IFACE+"EulerOperators" );
      t.newOutputPlug(    PL_WINDOW, IFACE+"GraphicsViewer" );
      t.newOptOutputPlug( PL_OUTPUT, IFACE+"Brep" );
        // Property:
      t.propBegin( SOLID, TYPE_INTEGER, "Construction mode/solid", true );
        t.propDefault( Constructions.TETRAHEDRON.ordinal() );
        t.propManipulator( MANIPULATOR_COMBO );
        t.propEnum( "Tetrahedron",    Constructions.TETRAHEDRON.ordinal(),    "Tetrahedron" );
        t.propEnum( "Cube",           Constructions.CUBE.ordinal(),           "Cube" );
        t.propEnum( "Graph (R->R^2)", Constructions.FUNCTION_GRAPH.ordinal(), "Graph of attached R->R^2 function" );
        t.propEnum( "XML script",     Constructions.XML_SCRIPT.ordinal(),     "Data read from XML script" );
      t.propEnd();
      t.propBegin( PERIOD, TYPE_INTEGER, "Auto-redraw period in milliseconds (0 for none)", true );
        t.propDefault( 10 );
        t.propBounds(  0, 60000 );
      t.propEnd();
    }
    return 1;
  }

  /**
   * Static registration instance for this class.
   * Automatically initialized in class-loading time.
   */
  public static final RegPiece reg;
  static
  {
    reg = new RegPiece();
    setTemplate( reg, 0 );
  }

}
